/**
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.deephacks.tools4j.config;

import java.util.List;

/**
 * ConfigQuery is used for fine-grained control over what instances are
 * fetched at runtime. 
 * 
 * This object is injectable on fields using the @Config annotation. 
 * 
 * ConfigQuery fields operate on all instance of a particiular config type 
 * and these fields are not configurable by an administrator. 
 * 
 * @param <T> A configurable bean type.  
 * 
 * @author Kristoffer Sjogren
 */
public abstract class ConfigQuery<T> {
    /**
     * @return ordered collection of all instance T sorted by id.
     */
    public abstract List<T> all();

    /**
     * List n number of instances of T in a ordered collection 
     * sorted by id. Calling next multiple time return instances
     * until eventually the end is reached and an empty collection
     * is returned.
     * 
     * @param num number of instance to return
     * @return a list of instances of T
     */
    public abstract List<T> next(int num);

    /**
     * Get a specific instance T with a certain id.
     * 
     * @param id of instance to get.
     * @return the instance of null if not found.
     */
    public abstract T get(Object id);
}
